<?php
/**
*
* @name acp_quickmod.php
* @package acp cBB QuickMod
* @version $Id: acp_quickmod.php,v1.0.3 20/07/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

/**
* @package acp
*/
class acp_quickmod
{
	var $u_action;
	var $new_config = array();

	function main($id, $mode)
	{
		global $phpbb_root_path, $user, $config, $template, $phpEx;

		if(!class_exists('quickmod'))
		{
			include($phpbb_root_path . 'quickmod/includes/quickmod.' . $phpEx);
			$quickmod = new quickmod(false);
		}

		$action = request_var('action', '');
		$submit = (isset($_POST['submit'])) ? true : false;
	
		$all_actions		= $quickmod->actions;
		$selected_actions	= explode(',', (isset($config['qm_buttons']) ? $config['qm_buttons'] : ''));
		$cfg_array			= (isset($_REQUEST['config'])) ? request_var('config', array('' => '')) : $config;
		$error				= array();

		$display_vars = array(
			'legend1'			=> 'QM_MAIN_CONFIG',
			'qm_enabled'		=> array('lang' => 'QM_ENABLED',	'validate' => 'bool',	'type' => 'radio:yes_no'),
			'qm_confirm'		=> array('lang' => 'QM_CONFIRM',	'validate' => 'bool',	'type' => 'radio:yes_no'),
			'qm_type'			=> array('lang' => 'QM_TYPE',		'validate' => 'string',	'type' => 'select',	'method' => 'make_type_select',	'params' => array('{CONFIG_VALUE}')),
		);

		// We validate the complete config if whished
		validate_config_vars($display_vars, $cfg_array, $error);
		
		// Do not write values if there is an error
		if(sizeof($error))
		{
			$submit = false;
		}
		
		// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
		foreach ($display_vars as $config_name => $null)
		{
			if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
			{
				continue;
			}

			$this->new_config[$config_name] = $config_value = $cfg_array[$config_name];

			if ($submit)
			{
				set_config($config_name, $config_value);
			}
		}

		if($submit)
		{
			$new_buttons = request_var('selected_buttons', array(0 => ''));
			
			set_config('qm_buttons', implode(',', $new_buttons));

			add_log('admin', 'LOG_QUICKMOD_CONFIG');
			trigger_error($user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}
		
		// Output relevant page
		foreach($display_vars as $config_key => $vars)
		{
			if(!is_array($vars) && strpos($config_key, 'legend') === false)
			{
				continue;
			}

			if(strpos($config_key, 'legend') !== false)
			{
				$template->assign_block_vars('options', array(
					'S_LEGEND'		=> true,
					'LEGEND'		=> (isset($user->lang[$vars])) ? $user->lang[$vars] : $vars)
				);

				continue;
			}

			$type 		= explode(':', $vars['type']);
			$content 	= build_cfg_template($type, $config_key, $this->new_config, $config_key, $vars);

			if(empty($content))
			{
				continue;
			}

			$template->assign_block_vars('options', array(
				'KEY'			=> $config_key,
				'TITLE'			=> (isset($user->lang[$vars['lang']])) ? $user->lang[$vars['lang']] : $vars['lang'],
				'S_EXPLAIN'		=> (isset($user->lang[$vars['lang'] . '_EXPLAIN'])) ? $user->lang[$vars['lang'] . '_EXPLAIN'] : '',
				'CONTENT'		=> $content,
			));

			unset($display_vars[$config_key]);
		}
	
		$button_row = array(
			'selected'	=> array(),
			'available'	=> array(),
		);
		
		foreach($selected_actions as $action)
		{
			if(!isset($all_actions[$action]))
			{
				continue;
			}
			
			$button_row['selected'][] = $action;
		}
		
		foreach($all_actions as $action => $row)
		{
			if(strpos($action, 'legend') !== false || (isset($row['in_button']) && !$row['in_button']) || in_array($action, $selected_actions))
			{
				continue;
			}
			
			$button_row['available'][] = $action;
		}
		
		foreach($button_row as $block_name => $actions)
		{
			foreach($actions as $action)
			{
				$row = $all_actions[$action];

				// Put block into general template to display
				$template->assign_block_vars($block_name . '_button', array(
					'S_TITLE'		=> $user->lang[$row['lang'] . '_TOPIC'],
					'S_ACTION'		=> $action,
	
					'S_HIDDEN_FIELDS'	=> '<input type="hidden" name="' . $block_name . '_buttons[]" value="' . $action . '" />',
				));
			}
		}

		$this->tpl_name 	= 'quickmod/acp_quickmod';
		$this->page_title	= 'QM_' . strtoupper($mode) . '_TITLE';

		// Check version
		$this->check_version_info();

		$template->assign_vars(array(
			'L_TITLE'			=> $user->lang[$this->page_title],
			'L_TITLE_EXPLAIN'	=> $user->lang['QM_' . strtoupper($mode) . '_EXPLAIN'],

			'S_ERROR'			=> (sizeof($error)) ? true : false,
			'ERROR_MSG'			=> implode('<br />', $error),

			'U_ACTION'			=> $this->u_action,
		));
	}
	
	function make_type_select($select_id)
	{
		global $user;
		
		$options = '';

		$ary = array(
			'icon'		=> 'QM_TYPE_ICON',
			'menu'		=> 'QM_TYPE_MENU',
		);
		
		foreach($ary as $value => $lang)
		{
			$selected = ($value == $select_id) ? ' selected="selected"' : '';
			$options .= '<option value="' . $value . '"' . $selected . '>' . $user->lang[$lang] . '</option>';
		}
		
		return $options;
	}

	/**
	 * Check the version information of cBB QuickMod
	 *
	 * @param bool $force_update Ignores cached data. Defaults to false.
	 * @param int $ttl Cache version information for $ttl seconds. Defaults to 259200 (72 hours).
	 *
	 */
	function check_version_info($ttl = 259200)
	{
		global $config, $template, $cache;
		
		$force_update 	= request_var('versioncheck_force', false);
		$info 			= $cache->get('qm_versioncheck');

		if($info === false || $force_update)
		{
			$errstr = '';
			$errno	= 0;

			$info = get_remote_file('www.canidev.com', '/app/info', 'cbb-quickmod.txt', $errstr, $errno);

			if($info === false || !preg_match('#^(\d+)\.(\d+)\.(\d+)#', $info))
			{
				$cache->destroy('qm_versioncheck');
				return false;
			}

			$cache->put('qm_versioncheck', $info, $ttl);
		}

		$current_version 		= (isset($config['qm_version']) ? $config['qm_version'] : '1.0.0');
		$latest_version_info 	= explode("\n", $info, 2);
		$latest_version			= trim($latest_version_info[0]);

		$template->assign_vars(array(
			'S_VERSION_NOT_UP_TO_DATE'	=> version_compare($current_version, $latest_version, '<') ? true : false,

			'U_VERSIONCHECK_FORCE'	=> $this->u_action . '&amp;versioncheck_force=1',
			'U_MORE_INFORMATION'	=> trim($latest_version_info[1]),
		));
	}
}
?>
