<?php
/*
* @name functions_quickmod.php
* @package phpBB3 cBB QuickMod
* @version $Id: functions_quickmod.php,v1.0.3 20/07/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
* Note: This file include some functions adapted from phpBB3 original admin/mcp to work property with this project.
*
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

// Rename Topic
function rename_topic($topic_ids, &$data)
{
	global $db, $phpbb_root_path, $phpEx, $config;

	if(empty($topic_ids) || sizeof($topic_ids) != 1)
	{
		return false;
	}

	$new_title = utf8_normalize_nfc(request_var('topic_title', '', true));

	// Get some data before changed
	$sql = 'SELECT t.forum_id, t.topic_id, t.topic_poster, t.topic_first_post_id, t.topic_first_poster_name, t.topic_moved_id,
		f.enable_indexing, p.post_text
		FROM ' . TOPICS_TABLE . ' t
			LEFT JOIN ' . FORUMS_TABLE . ' f ON(f.forum_id = t.forum_id)
			LEFT JOIN ' . POSTS_TABLE . ' p ON (p.post_id = t.topic_first_post_id)
		WHERE t.topic_id = ' . $topic_ids[0];
	$result = $db->sql_query($sql);
	$topic_data = $db->sql_fetchrow($result);
	$db->sql_freeresult($result);

	if(!$topic_data || !$new_title)
	{
		return false;
	}

	// Update the topic
	$sql = 'UPDATE ' . TOPICS_TABLE . "
		SET topic_title = '" . $db->sql_escape($new_title) . "'
		WHERE topic_id = " . $topic_data['topic_id'] .
		(($topic_data['topic_moved_id']) ? " OR topic_id = {$topic_data['topic_moved_id']} OR topic_moved_id = {$topic_data['topic_moved_id']}" : "");
	$db->sql_query($sql);

	// Update the first post
	$sql = 'UPDATE ' . POSTS_TABLE . "
		SET post_subject = '" . $db->sql_escape($new_title) . "'
		WHERE post_id = " . $topic_data['topic_first_post_id'];
	$db->sql_query($sql);
	
	// Update the last post subject in forums
	$sql = 'UPDATE ' . FORUMS_TABLE . "
		SET forum_last_post_subject = '" . $db->sql_escape($new_title) . "'
		WHERE forum_last_post_id = " . $topic_data['topic_first_post_id'];
	$db->sql_query($sql);
	
	// Index message contents
	if(!empty($topic_data['enable_indexing']))
	{
		// Select the search method and do some additional checks to ensure it can actually be utilised
		$search_type = basename($config['search_type']);

		if(file_exists($phpbb_root_path . 'includes/search/' . $search_type . '.' . $phpEx))
		{
			if(!class_exists($search_type))
			{
				include("{$phpbb_root_path}includes/search/$search_type.$phpEx");
			}

			$error = false;
			$search = new $search_type($error);

			if(!$error)
			{
				$search->index('edit', $topic_data['topic_first_post_id'], $topic_data['post_text'], $new_title, $topic_data['topic_poster'], $topic_data['forum_id']);
			}
			
			unset($search);
		}
	}

	add_log('mod', $topic_data['forum_id'], $topic_data['topic_id'], 'LOG_POST_EDITED', $new_title, $topic_data['topic_first_poster_name']);

	$data = htmlspecialchars_decode($new_title);

	return 'TOPIC_RENAMED_SUCCESS';
}

/**
*
* Functions based on phpBB3 original functions but adapted to ajax request and data return
* (the original functions is invalid for this)
*
*/

// Lock/Unlock Topic
function lock_unlock($action, $ids)
{
	global $auth, $user, $db, $phpEx, $phpbb_root_path;

	if(($action != 'lock' && $action != 'unlock') || !is_array($ids))
	{
		return false;
	}

	$orig_ids = $ids;

	if (!check_ids($ids, TOPICS_TABLE, 'topic_id', array('m_lock')))
	{
		// Make sure that for f_user_lock only the lock action is triggered.
		if ($action != 'lock')
		{
			return false;
		}

		$ids = $orig_ids;

		if (!check_ids($ids, TOPICS_TABLE, 'topic_id', array('f_user_lock')))
		{
			return false;
		}
	}
	unset($orig_ids);

	$sql = 'UPDATE ' . TOPICS_TABLE . '
		SET topic_status = ' . (($action == 'lock') ? ITEM_LOCKED : ITEM_UNLOCKED) . '
		WHERE ' . $db->sql_in_set('topic_id', $ids) . '
		AND topic_moved_id = 0';
	$db->sql_query($sql);

	$data = get_topic_data($ids);

	foreach($data as $id => $row)
	{
		if(!$row['topic_moved_id'])
		{
			add_log('mod', $row['forum_id'], $row['topic_id'], 'LOG_' . strtoupper($action), $row['topic_title']);
		}
	}

	return ((sizeof($ids) == 1) ? 'TOPIC' : 'TOPICS') . '_' . (($action == 'lock') ? 'LOCKED' : 'UNLOCKED') . '_SUCCESS';
}


// Change Topic Type
function change_topic_type($action, $topic_ids, $forum_id = 0, $global_involved = false)
{
	global $auth, $user, $db, $phpEx, $phpbb_root_path, $template;

	$topic_sync = '';

	switch ($action)
	{
		case 'make_announce':
			$new_topic_type = POST_ANNOUNCE;
			$check_acl = 'f_announce';
		break;

		case 'make_global':
			$new_topic_type = POST_GLOBAL;
			$check_acl = 'f_announce';
		break;

		case 'make_sticky':
			$new_topic_type = POST_STICKY;
			$check_acl = 'f_sticky';
		break;

		default:
			$new_topic_type = POST_NORMAL;
			$check_acl = '';
		break;
	}

	// Exclude shadow topics
	$data = get_topic_data($topic_ids);

	foreach($data as $topic_id => $row)
	{
		if ($row['topic_moved_id'])
		{
			if(($id = array_search($topic_id, $topic_ids, true)) !== false)
			{
				unset($topic_ids[$id]);
			}
		}
	}

	if(empty($topic_ids))
	{
		return false;
	}

	if ($new_topic_type != POST_GLOBAL)
	{
		$sql = 'UPDATE ' . TOPICS_TABLE . "
			SET topic_type = $new_topic_type
			WHERE " . $db->sql_in_set('topic_id', $topic_ids) . '
				AND forum_id <> 0';
		$db->sql_query($sql);
		
		// Reset forum id if a global topic is within the array
		$to_forum_id = request_var('to_forum_id', $forum_id);

		if($to_forum_id)
		{
			$sql = 'UPDATE ' . TOPICS_TABLE . "
				SET topic_type = $new_topic_type, forum_id = $to_forum_id
				WHERE " . $db->sql_in_set('topic_id', $topic_ids) . '
					AND forum_id = 0';
			$db->sql_query($sql);

			// Update forum_ids for all posts
			$sql = 'UPDATE ' . POSTS_TABLE . "
				SET forum_id = $to_forum_id
				WHERE " . $db->sql_in_set('topic_id', $topic_ids) . '
					AND forum_id = 0';
			$db->sql_query($sql);

			// Do a little forum sync stuff
			$sql = 'SELECT SUM(t.topic_replies + t.topic_approved) as topic_posts, COUNT(t.topic_approved) as topics_authed
				FROM ' . TOPICS_TABLE . ' t
				WHERE ' . $db->sql_in_set('t.topic_id', $topic_ids);
			$result = $db->sql_query($sql);
			$row_data = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			$sync_sql = array();

			if ($row_data['topic_posts'])
			{
				$sync_sql[$to_forum_id][]	= 'forum_posts = forum_posts + ' . (int) $row_data['topic_posts'];
			}

			if ($row_data['topics_authed'])
			{
				$sync_sql[$to_forum_id][]	= 'forum_topics = forum_topics + ' . (int) $row_data['topics_authed'];
			}

			$sync_sql[$to_forum_id][]	= 'forum_topics_real = forum_topics_real + ' . (int) sizeof($topic_ids);

			foreach ($sync_sql as $forum_id_key => $array)
			{
				$sql = 'UPDATE ' . FORUMS_TABLE . '
					SET ' . implode(', ', $array) . '
					WHERE forum_id = ' . $forum_id_key;
				$db->sql_query($sql);
			}

			sync('forum', 'forum_id', $to_forum_id);
		}
	}
	else
	{
		// Get away with those topics already being a global announcement by re-calculating $topic_ids
		$sql = 'SELECT topic_id
			FROM ' . TOPICS_TABLE . '
			WHERE ' . $db->sql_in_set('topic_id', $topic_ids) . '
				AND forum_id <> 0';
		$result = $db->sql_query($sql);

		$topic_ids = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$topic_ids[] = $row['topic_id'];
		}
		$db->sql_freeresult($result);

		if (sizeof($topic_ids))
		{
			// Delete topic shadows for global announcements
			$sql = 'DELETE FROM ' . TOPICS_TABLE . '
				WHERE ' . $db->sql_in_set('topic_moved_id', $topic_ids);
			$db->sql_query($sql);

			$sql = 'UPDATE ' . TOPICS_TABLE . "
				SET topic_type = $new_topic_type, forum_id = 0
					WHERE " . $db->sql_in_set('topic_id', $topic_ids);
			$db->sql_query($sql);

			// Update forum_ids for all posts
			$sql = 'UPDATE ' . POSTS_TABLE . '
				SET forum_id = 0
				WHERE ' . $db->sql_in_set('topic_id', $topic_ids);
			$db->sql_query($sql);

			// Do a little forum sync stuff
			$sql = 'SELECT SUM(t.topic_replies + t.topic_approved) as topic_posts, COUNT(t.topic_approved) as topics_authed
				FROM ' . TOPICS_TABLE . ' t
				WHERE ' . $db->sql_in_set('t.topic_id', $topic_ids);
			$result = $db->sql_query($sql);
			$row_data = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			$sync_sql = array();

			if ($row_data['topic_posts'])
			{
				$sync_sql[$forum_id][]	= 'forum_posts = forum_posts - ' . (int) $row_data['topic_posts'];
			}

			if ($row_data['topics_authed'])
			{
				$sync_sql[$forum_id][]	= 'forum_topics = forum_topics - ' . (int) $row_data['topics_authed'];
			}

			$sync_sql[$forum_id][]	= 'forum_topics_real = forum_topics_real - ' . (int) sizeof($topic_ids);

			foreach ($sync_sql as $forum_id_key => $array)
			{
				$sql = 'UPDATE ' . FORUMS_TABLE . '
					SET ' . implode(', ', $array) . '
					WHERE forum_id = ' . $forum_id_key;
				$db->sql_query($sql);
			}

			sync('forum', 'forum_id', $forum_id);
		}
	}

	if (sizeof($topic_ids))
	{
		$data = get_topic_data($topic_ids);

		foreach ($data as $topic_id => $row)
		{
			add_log('mod', $forum_id, $topic_id, 'LOG_TOPIC_TYPE_CHANGED', $row['topic_title']);
		}
	}

	return (sizeof($topic_ids) == 1) ? 'TOPIC_TYPE_CHANGED' : 'TOPICS_TYPE_CHANGED';
}


// Delete Topics
function mcp_delete_topic($topic_ids)
{
	global $auth, $user, $db, $phpEx, $phpbb_root_path;

	if (!check_ids($topic_ids, TOPICS_TABLE, 'topic_id', array('m_delete')))
	{
		return false;
	}

	$data = get_topic_data($topic_ids);

	foreach ($data as $topic_id => $row)
	{
		if ($row['topic_moved_id'])
		{
			add_log('mod', $row['forum_id'], $topic_id, 'LOG_DELETE_SHADOW_TOPIC', $row['topic_title']);
		}
		else
		{
			add_log('mod', $row['forum_id'], $topic_id, 'LOG_DELETE_TOPIC', $row['topic_title'], $row['topic_first_poster_name']);
		}
	}

	delete_topics('topic_id', $topic_ids);

	return (sizeof($topic_ids) == 1) ? 'TOPIC_DELETED_SUCCESS' : 'TOPICS_DELETED_SUCCESS';
}

// Move Topic
function mcp_move_topic($topic_ids, $forum_id)
{
	global $auth, $user, $db, $template, $phpEx, $phpbb_root_path;
	global $quick_mod;

	$to_forum_id = request_var('to_forum_id', 0);
	$additional_msg = $success_msg = '';

	if($to_forum_id)
	{
		$forum_data = get_forum_data($to_forum_id, 'f_post');

		if (!sizeof($forum_data))
		{
			$quick_mod->send('FORUM_NOT_EXIST', true);
		}
		else
		{
			$forum_data = $forum_data[$to_forum_id];

			if ($forum_data['forum_type'] != FORUM_POST)
			{
				$quick_mod->send('FORUM_NOT_POSTABLE', true);
			}
			else if (!$auth->acl_get('f_post', $to_forum_id) || (!$auth->acl_get('m_approve', $to_forum_id) && !$auth->acl_get('f_noapprove', $to_forum_id)))
			{
				$quick_mod->send('USER_CANNOT_POST', true);
			}
			else if ($forum_id == $to_forum_id)
			{
				$quick_mod->send('CANNOT_MOVE_SAME_FORUM', true);
			}
		}
	}
	else
	{
		$quick_mod->send('FORUM_NOT_EXIST', true);
	}

	$topic_data = get_topic_data($topic_ids, true);
	$leave_shadow = (isset($_POST['move_leave_shadow'])) ? true : false;

	$forum_sync_data = array();

	$forum_sync_data[$forum_id] = current($topic_data);
	$forum_sync_data[$to_forum_id] = $forum_data;

	// Real topics added to target forum
	$topics_moved = sizeof($topic_data);

	// Approved topics added to target forum
	$topics_authed_moved = 0;

	// Posts (topic replies + topic post if approved) added to target forum
	$topic_posts_added = 0;

	// Posts (topic replies + topic post if approved and not global announcement) removed from source forum
	$topic_posts_removed = 0;

	// Real topics removed from source forum (all topics without global announcements)
	$topics_removed = 0;

	// Approved topics removed from source forum (except global announcements)
	$topics_authed_removed = 0;

	foreach($topic_data as $topic_id => $topic_info)
	{
		// If is a shadow topic, skip
		if($topic_info['topic_moved_id'])
		{
			unset($topic_data[$topic_id]);

			if(($id = array_search($topic_id, $topic_ids, true)) !== false)
			{
				unset($topic_ids[$id]);
			}
			continue;
		}

		if($topic_info['topic_approved'])
		{
			$topics_authed_moved++;
			$topic_posts_added++;
		}

		$topic_posts_added += $topic_info['topic_replies'];

		if($topic_info['topic_type'] != POST_GLOBAL)
		{
			$topics_removed++;
			$topic_posts_removed += $topic_info['topic_replies'];

			if($topic_info['topic_approved'])
			{
				$topics_authed_removed++;
				$topic_posts_removed++;
			}
		}
	}

	if(empty($topic_ids))
	{
		return false;
	}

	$db->sql_transaction('begin');

	$sync_sql = array();

	if($topic_posts_added)
	{
		$sync_sql[$to_forum_id][] = 'forum_posts = forum_posts + ' . $topic_posts_added;
	}

	if($topics_authed_moved)
	{
		$sync_sql[$to_forum_id][] = 'forum_topics = forum_topics + ' . (int) $topics_authed_moved;
	}

	$sync_sql[$to_forum_id][] = 'forum_topics_real = forum_topics_real + ' . (int) $topics_moved;

	// Move topics, but do not resync yet
	move_topics($topic_ids, $to_forum_id, false);

	$forum_ids = array($to_forum_id);
	foreach($topic_data as $topic_id => $row)
	{
		// Get the list of forums to resync, add a log entry
		$forum_ids[] = $row['forum_id'];
		add_log('mod', $to_forum_id, $topic_id, 'LOG_MOVE', $row['forum_name'], $forum_data['forum_name']);

		// If we have moved a global announcement, we need to correct the topic type
		if($row['topic_type'] == POST_GLOBAL)
		{
			$sql = 'UPDATE ' . TOPICS_TABLE . '
				SET topic_type = ' . POST_ANNOUNCE . '
				WHERE topic_id = ' . (int) $row['topic_id'];
			$db->sql_query($sql);
		}

		// Leave a redirection if required and only if the topic is visible to users
		if($leave_shadow && $row['topic_approved'] && $row['topic_type'] != POST_GLOBAL)
		{
			$shadow = array(
				'forum_id'				=>	(int) $row['forum_id'],
				'icon_id'				=>	(int) $row['icon_id'],
				'topic_attachment'		=>	(int) $row['topic_attachment'],
				'topic_approved'		=>	1, // a shadow topic is always approved
				'topic_reported'		=>	0, // a shadow topic is never reported
				'topic_title'			=>	(string) $row['topic_title'],
				'topic_poster'			=>	(int) $row['topic_poster'],
				'topic_time'			=>	(int) $row['topic_time'],
				'topic_time_limit'		=>	(int) $row['topic_time_limit'],
				'topic_views'			=>	(int) $row['topic_views'],
				'topic_replies'			=>	(int) $row['topic_replies'],
				'topic_replies_real'	=>	(int) $row['topic_replies_real'],
				'topic_status'			=>	ITEM_MOVED,
				'topic_type'			=>	POST_NORMAL,
				'topic_first_post_id'	=>	(int) $row['topic_first_post_id'],
				'topic_first_poster_colour'=>(string) $row['topic_first_poster_colour'],
				'topic_first_poster_name'=>	(string) $row['topic_first_poster_name'],
				'topic_last_post_id'	=>	(int) $row['topic_last_post_id'],
				'topic_last_poster_id'	=>	(int) $row['topic_last_poster_id'],
				'topic_last_poster_colour'=>(string) $row['topic_last_poster_colour'],
				'topic_last_poster_name'=>	(string) $row['topic_last_poster_name'],
				'topic_last_post_subject'=>	(string)  $row['topic_last_post_subject'],
				'topic_last_post_time'	=>	(int) $row['topic_last_post_time'],
				'topic_last_view_time'	=>	(int) $row['topic_last_view_time'],
				'topic_moved_id'		=>	(int) $row['topic_id'],
				'topic_bumped'			=>	(int) $row['topic_bumped'],
				'topic_bumper'			=>	(int) $row['topic_bumper'],
				'poll_title'			=>	(string) $row['poll_title'],
				'poll_start'			=>	(int) $row['poll_start'],
				'poll_length'			=>	(int) $row['poll_length'],
				'poll_max_options'		=>	(int) $row['poll_max_options'],
				'poll_last_vote'		=>	(int) $row['poll_last_vote']
			);

			$db->sql_query('INSERT INTO ' . TOPICS_TABLE . $db->sql_build_array('INSERT', $shadow));

			// Shadow topics only count on new "topics" and not posts... a shadow topic alone has 0 posts
			$topics_removed--;
			$topics_authed_removed--;
		}
	}
	unset($topic_data);

	if($topic_posts_removed)
	{
		$sync_sql[$forum_id][] = 'forum_posts = forum_posts - ' . $topic_posts_removed;
	}

	if($topics_removed)
	{
		$sync_sql[$forum_id][]	= 'forum_topics_real = forum_topics_real - ' . (int) $topics_removed;
	}

	if($topics_authed_removed)
	{
		$sync_sql[$forum_id][]	= 'forum_topics = forum_topics - ' . (int) $topics_authed_removed;
	}

	foreach($sync_sql as $forum_id_key => $array)
	{
		$sql = 'UPDATE ' . FORUMS_TABLE . '
			SET ' . implode(', ', $array) . '
			WHERE forum_id = ' . $forum_id_key;
		$db->sql_query($sql);
	}

	$db->sql_transaction('commit');

	sync('forum', 'forum_id', array($forum_id, $to_forum_id));

	return (sizeof($topic_ids) == 1) ? 'TOPIC_MOVED_SUCCESS' : 'TOPICS_MOVED_SUCCESS';
}

function approve_topic($action, $topic_ids)
{
	global $db, $phpbb_root_path, $phpEx;

	include($phpbb_root_path . 'includes/functions_messenger.' . $phpEx);
	include($phpbb_root_path . 'includes/functions_posting.' . $phpEx);

	$notify_poster = (isset($_POST['send_notify']) ? true : false);

	$sql = 'SELECT f.forum_name, t.forum_id, t.topic_id, t.topic_title, t.topic_first_post_id, t.topic_poster, t.topic_replies, t.topic_moved_id,
		u.username, u.user_lang, u.user_email, u.user_jabber, u.user_notify_type
		FROM ' . TOPICS_TABLE . ' t
		LEFT JOIN ' . FORUMS_TABLE . ' f ON(f.forum_id = t.forum_id)
		LEFT JOIN ' . USERS_TABLE . ' u ON(u.user_id = t.topic_poster)
		WHERE ' . $db->sql_in_set('t.topic_id', $topic_ids) . '
			AND t.topic_approved = 0';
	$result = $db->sql_query($sql);

	$post_ids = $topic_data = $user_ary = array();
	$topic_id_list = $forum_id_list = array();
	$total_topics = $total_posts = 0;

	while($row = $db->sql_fetchrow($result))
	{
		if($row['topic_moved_id'])
		{
			continue;
		}

		$post_ids[] = $row['topic_first_post_id'];
		$topic_data[] = $row;

		$user_ary[$row['topic_poster']] = (empty($user_ary[$row['topic_poster']])) ? 1 : $user_ary[$row['topic_poster']] + 1;

		$topic_id_list[] = $row['topic_id'];

		if ($row['forum_id'])
		{
			$forum_id_list[$row['forum_id']] = 1;
		}

		if($row['forum_id'])
		{
			$total_topics++;
			$total_posts++;

			// Increment by topic_replies if we approve a topic...
			// This works because we do not adjust the topic_replies when re-approving a topic after an edit.
			if($row['topic_replies'])
			{
				$total_posts += $row['topic_replies'];
			}
		}
	}
	$db->sql_freeresult($result);

	if (!sizeof($post_ids))
	{
		return false;
	}

	$sql = 'UPDATE ' . TOPICS_TABLE . '
		SET topic_approved = 1
		WHERE ' . $db->sql_in_set('topic_id', $topic_id_list);
	$db->sql_query($sql);

	$sql = 'UPDATE ' . POSTS_TABLE . '
		SET post_approved = 1
		WHERE ' . $db->sql_in_set('post_id', $post_ids);
	$db->sql_query($sql);

	foreach($topic_data as $data)
	{
		add_log('mod', $data['forum_id'], $data['topic_id'], 'LOG_TOPIC_APPROVED', $data['topic_title']);
	}

	if (sizeof($user_ary))
	{
		// Try to minimize the query count by merging users with the same post count additions
		$user_posts_update = array();

		foreach ($user_ary as $user_id => $user_posts)
		{
			$user_posts_update[$user_posts][] = $user_id;
		}

		foreach ($user_posts_update as $user_posts => $user_id_ary)
		{
			$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_posts = user_posts + ' . $user_posts . '
				WHERE ' . $db->sql_in_set('user_id', $user_id_ary);
			$db->sql_query($sql);
		}
	}

	if ($total_topics)
	{
		set_config_count('num_topics', $total_topics, true);
	}

	if ($total_posts)
	{
		set_config_count('num_posts', $total_posts, true);
	}

	sync('topic', 'topic_id', $topic_id_list, true);
	sync('forum', 'forum_id', array_keys($forum_id_list), true, true);
	unset($topic_id_list, $forum_id_list);

	$messenger = new messenger();

	// Notify Poster?
	if($notify_poster)
	{
		foreach($topic_data as $data)
		{
			if ($data['topic_poster'] == ANONYMOUS)
			{
				continue;
			}

			$messenger->template('topic_approved', $data['user_lang']);

			$messenger->to($data['user_email'], $data['username']);
			$messenger->im($data['user_jabber'], $data['username']);

			$messenger->assign_vars(array(
				'USERNAME'		=> htmlspecialchars_decode($data['username']),
				'TOPIC_TITLE'	=> htmlspecialchars_decode(censor_text($data['topic_title'])),

				'U_VIEW_TOPIC'	=> generate_board_url() . "/viewtopic.$phpEx?f={$data['forum_id']}&t={$data['topic_id']}&e=0",
			));

			$messenger->send($data['user_notify_type']);
		}
	}

	$messenger->save_queue();

	foreach($topic_data as $data)
	{
		// Forum Notifications
		user_notification('post', $data['topic_title'], $data['topic_title'], $data['forum_name'], $data['forum_id'], $data['topic_id'], $data['topic_first_post_id']);
	}

	return ($total_topics == 1) ? 'TOPIC_APPROVED_SUCCESS' : 'TOPICS_APPROVED_SUCCESS';
}

function qm_chgposter($topic_ids)
{
	global $db, $auth, $phpbb_root_path, $phpEx;

	$post_ids = array();
	$username = request_var('username', '', true);
	
	$sql = 'SELECT *
		FROM ' . USERS_TABLE . "
		WHERE username_clean = '" . $db->sql_escape(utf8_clean_string($username)) . "'";
	$result = $db->sql_query($sql);
	$userdata = $db->sql_fetchrow($result);
	$db->sql_freeresult($result);
	
	if (!$userdata)
	{
		return false;
	}
	
	// Get the posts ids
	$sql = 'SELECT topic_first_post_id
		FROM ' . TOPICS_TABLE . '
		WHERE ' . $db->sql_in_set('topic_id', $topic_ids);
	$result = $db->sql_query($sql);
	while($row = $db->sql_fetchrow($result))
	{
		$post_ids[] = $row['topic_first_post_id'];
	}
	$db->sql_freeresult($result);

	if(!empty($post_ids))
	{
		// Get posts data
		$posts_info = get_post_data($post_ids, false, true);
		
		// Update all posts
		$sql = 'UPDATE ' . POSTS_TABLE . '
			SET poster_id = ' . $userdata['user_id'] . '
			WHERE ' . $db->sql_in_set('post_id', $post_ids);
		$db->sql_query($sql);
		
		// sync the posts info
		foreach($posts_info as $post_id => $post_info)
		{
			qm_change_poster($post_info, $userdata);
		}
		
		return 'AUTHOR_CHANGE_SUCCESS';
	}
	
	return false;
}

/**
* Change a post's poster
*/
function qm_change_poster($post_info, $userdata)
{
	global $auth, $db, $config, $phpbb_root_path, $phpEx;

	if (empty($userdata) || $userdata['user_id'] == $post_info['user_id'])
	{
		return;
	}

	$post_id = $post_info['post_id'];

	// Resync topic/forum if needed
	if ($post_info['topic_last_post_id'] == $post_id || $post_info['forum_last_post_id'] == $post_id || $post_info['topic_first_post_id'] == $post_id)
	{
		sync('topic', 'topic_id', $post_info['topic_id'], false, false);
		sync('forum', 'forum_id', $post_info['forum_id'], false, false);
	}

	// Adjust post counts... only if the post is approved (else, it was not added the users post count anyway)
	if ($post_info['post_postcount'] && $post_info['post_approved'])
	{
		$sql = 'UPDATE ' . USERS_TABLE . '
			SET user_posts = user_posts - 1
			WHERE user_id = ' . $post_info['user_id'] .'
			AND user_posts > 0';
		$db->sql_query($sql);

		$sql = 'UPDATE ' . USERS_TABLE . '
			SET user_posts = user_posts + 1
			WHERE user_id = ' . $userdata['user_id'];
		$db->sql_query($sql);
	}

	// Add posted to information for this topic for the new user
	markread('post', $post_info['forum_id'], $post_info['topic_id'], time(), $userdata['user_id']);

	// Remove the dotted topic option if the old user has no more posts within this topic
	if ($config['load_db_track'] && $post_info['user_id'] != ANONYMOUS)
	{
		$sql = 'SELECT topic_id
			FROM ' . POSTS_TABLE . '
			WHERE topic_id = ' . $post_info['topic_id'] . '
				AND poster_id = ' . $post_info['user_id'];
		$result = $db->sql_query_limit($sql, 1);
		$topic_id = (int) $db->sql_fetchfield('topic_id');
		$db->sql_freeresult($result);

		if (!$topic_id)
		{
			$sql = 'DELETE FROM ' . TOPICS_POSTED_TABLE . '
				WHERE user_id = ' . $post_info['user_id'] . '
					AND topic_id = ' . $post_info['topic_id'];
			$db->sql_query($sql);
		}
	}

	// change the poster_id within the attachments table, else the data becomes out of sync and errors displayed because of wrong ownership
	if ($post_info['post_attachment'])
	{
		$sql = 'UPDATE ' . ATTACHMENTS_TABLE . '
			SET poster_id = ' . $userdata['user_id'] . '
			WHERE poster_id = ' . $post_info['user_id'] . '
				AND post_msg_id = ' . $post_info['post_id'] . '
				AND topic_id = ' . $post_info['topic_id'];
		$db->sql_query($sql);
	}

	// refresh search cache of this post
	$search_type = basename($config['search_type']);

	if (file_exists($phpbb_root_path . 'includes/search/' . $search_type . '.' . $phpEx))
	{
		if(!class_exists($search_type))
		{
			require("{$phpbb_root_path}includes/search/$search_type.$phpEx");
		}

		// We do some additional checks in the module to ensure it can actually be utilised
		$error = false;
		$search = new $search_type($error);

		if (!$error && method_exists($search, 'destroy_cache'))
		{
			$search->destroy_cache(array(), array($post_info['user_id'], $userdata['user_id']));
		}
	}

	// Now add log entry
	add_log('mod', $post_info['forum_id'], $post_info['topic_id'], 'LOG_MCP_CHANGE_POSTER', $post_info['topic_title'], $post_info['username'], $userdata['username']);
}

function qm_chgicon($topic_ids)
{
	global $db;

	$post_ids = array();
	$icon_id = request_var('icon', 0);
	
	// Get the posts ids
	$sql = 'SELECT topic_first_post_id
		FROM ' . TOPICS_TABLE . '
		WHERE ' . $db->sql_in_set('topic_id', $topic_ids);
	$result = $db->sql_query($sql);
	while($row = $db->sql_fetchrow($result))
	{
		$post_ids[] = $row['topic_first_post_id'];
	}
	$db->sql_freeresult($result);
	
	// Update the topic's icons
	$sql = 'UPDATE ' . TOPICS_TABLE . '
		SET icon_id = ' . $icon_id . '
		WHERE ' . $db->sql_in_set('topic_id', $topic_ids);
	$db->sql_query($sql);

	// Update the first post's icons
	if(!empty($post_ids))
	{
		$sql = 'UPDATE ' . POSTS_TABLE . '
			SET icon_id = ' . $icon_id . '
			WHERE ' . $db->sql_in_set('post_id', $post_ids);
		$db->sql_query($sql);
	}
	
	return 'ICON_CHANGE_SUCCESS';
}

function qm_chgtime($topic_ids)
{
	global $db;
	
	$now = getdate();
	
	$new_day	= request_var('day', $now['mday']);
	$new_month	= request_var('month', $now['mon']);
	$new_year	= request_var('year', $now['year']);
	$new_hour	= request_var('hour', '00:00:00');
	
	$new_date = sprintf('%d/%d/%d %s', $new_month, $new_day, $new_year, $new_hour);

	$new_time = @strtotime($new_date);
		
	if($new_time === false || $new_time === -1)
	{
		return false;
	}

	$post_ids = $updated_forums = $last_post_ids = array();
	$last_topic_id = $timeadd = 0;
	
	// Get some neccessary data before continue
	$sql = 'SELECT p.post_id, p.topic_id, p.forum_id, p.poster_id, p.post_subject, p.post_time, p.post_approved,
		u.username, u.user_colour
		FROM ' . POSTS_TABLE . ' p
			LEFT JOIN ' . USERS_TABLE . ' u ON(u.user_id = p.poster_id)
		WHERE ' . $db->sql_in_set('p.topic_id', $topic_ids) . '
		ORDER BY p.topic_id ASC, p.post_time ASC';
	$result = $db->sql_query($sql);
	while($row = $db->sql_fetchrow($result))
	{
		$topic_id = (int)$row['topic_id'];
		$forum_id = (int)$row['forum_id'];

		if($row['post_time'] < $new_time)
		{
			// Add 1 minute to the last time
			$timeadd += 60;

			if($last_topic_id != $topic_id)
			{
				$last_topic_id = $topic_id;
				$time_add = 0;
			}
			
			// Maintain a time difference to keep post order
			$row['post_time'] = $new_time + $timeadd;
			
			$post_ids[$row['post_id']] = $row['post_time'];
		}
		
		// Maintain the order for the last post if required update
		if($row['post_approved'])
		{
			if(!isset($updated_forums[$forum_id]) || (isset($updated_forums[$forum_id]) && $updated_forums[$forum_id]['post_time'] < $row['post_time']))
			{
				$updated_forums[$forum_id] = $row;
			}
		}
	}
	$db->sql_freeresult($result);
	
	// Obtain the time of the last posts on involved forums
	$sql = 'SELECT forum_last_post_time
		FROM ' . FORUMS_TABLE . '
		WHERE ' . $db->sql_in_set('forum_id', array_keys($updated_forums));
	$result = $db->sql_query($sql);
	while($row = $db->sql_fetchrow($result))
	{
		$forum_id = (int)$row['forum_id'];
		$updated_forums[$forum_id]['forum_last_post_time'] = $row['forum_last_post_time'];
	}
	$db->sql_freeresult($result);
	
	// Set the new time for the first posts
	$sql = 'SELECT topic_first_post_id, topic_last_post_id
		FROM ' . TOPICS_TABLE . '
		WHERE ' . $db->sql_in_set('topic_id', $topic_ids);
	$result = $db->sql_query($sql);
	while($row = $db->sql_fetchrow($result))
	{
		$post_ids[$row['topic_first_post_id']] = $new_time;
		$last_post_ids[] = $row['topic_last_post_id'];
	}
	$db->sql_freeresult($result);
	
	// Update the time of the posts and topics
	$sql = 'UPDATE ' . TOPICS_TABLE . "
		SET topic_time = $new_time
		WHERE " . $db->sql_in_set('topic_id', $topic_ids);
	$db->sql_query($sql);

	foreach($post_ids as $post_id => $post_time)
	{
		$sql = 'UPDATE ' . POSTS_TABLE . "
			SET post_time = $post_time
			WHERE post_id = $post_id";
		$db->sql_query($sql);
		
		// Update the time of last post in topic
		if(in_array($post_id, $last_post_ids))
		{
			$sql = 'UPDATE ' . TOPICS_TABLE . "
				SET topic_last_post_time = $post_time
				WHERE topic_last_post_id = $post_id";
			$db->sql_query($sql);
		}
	}
	
	// Update last post id and time of forums if neccessary
	foreach($updated_forums as $forum_id => $row)
	{
		if($row['forum_last_post_time'] < $new_time)
		{
			$sql_ary = array(
				'forum_last_post_id'		=> $row['post_id'],
				'forum_last_poster_id'		=> $row['poster_id'],
				'forum_last_post_subject'	=> $row['post_subject'],
				'forum_last_post_time'		=> $row['post_time'],
				'forum_last_poster_name'	=> $row['username'],
				'forum_last_poster_colour'	=> $row['user_colour'],
			);
			
			$sql = 'UPDATE ' . FORUMS_TABLE . '
				SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
				WHERE forum_id = ' . $forum_id;
			$db->sql_query($sql);
		}
	}
	
	return 'TIME_CHANGE_SUCCESS';
}

function get_topic_data($ids, $full = false)
{
	global $db;

	$topic_row = array();

	if($full)
	{
		$sql = 'SELECT t.*, f.forum_name
			FROM ' . TOPICS_TABLE . ' t
			LEFT JOIN ' . FORUMS_TABLE . ' f ON(f.forum_id = t.forum_id)';
	}
	else
	{
		$sql = 'SELECT t.* FROM ' . TOPICS_TABLE . ' t';
	}

	$sql .= ' WHERE ' . $db->sql_in_set('t.topic_id', $ids);

	$result = $db->sql_query($sql);
	while($row = $db->sql_fetchrow($result))
	{
		$topic_row[$row['topic_id']] = $row;
	}

	return $topic_row;
}

/**
* Get simple forum data
*/
function get_forum_data($forum_id, $acl_list = 'f_list', $read_tracking = false)
{
	global $auth, $db, $user, $config;

	$rowset = array();

	if (!is_array($forum_id))
	{
		$forum_id = array($forum_id);
	}

	if (!sizeof($forum_id))
	{
		return array();
	}

	if ($read_tracking && $config['load_db_lastread'])
	{
		$read_tracking_join = ' LEFT JOIN ' . FORUMS_TRACK_TABLE . ' ft ON (ft.user_id = ' . $user->data['user_id'] . '
			AND ft.forum_id = f.forum_id)';
		$read_tracking_select = ', ft.mark_time';
	}
	else
	{
		$read_tracking_join = $read_tracking_select = '';
	}

	$sql = "SELECT f.* $read_tracking_select
		FROM " . FORUMS_TABLE . " f$read_tracking_join
		WHERE " . $db->sql_in_set('f.forum_id', $forum_id);
	$result = $db->sql_query($sql);

	while ($row = $db->sql_fetchrow($result))
	{
		if ($acl_list && !$auth->acl_gets($acl_list, $row['forum_id']))
		{
			continue;
		}

		if ($auth->acl_get('m_approve', $row['forum_id']))
		{
			$row['forum_topics'] = $row['forum_topics_real'];
		}

		$rowset[$row['forum_id']] = $row;
	}
	$db->sql_freeresult($result);

	return $rowset;
}

/**
* Validate ids
*
* @param	array	&$ids			The relevant ids to check
* @param	string	$table			The table to find the ids in
* @param	string	$sql_id			The ids relevant column name
* @param	array	$acl_list		A list of permissions the user need to have
*
* @return	mixed	False if no ids were able to be retrieved, true if at least one id left.
*
*/
function check_ids(&$ids, $table, $sql_id, $acl_list = false)
{
	global $db, $auth;

	if (!is_array($ids) || empty($ids))
	{
		return false;
	}

	$sql = "SELECT $sql_id, forum_id FROM $table
		WHERE " . $db->sql_in_set($sql_id, $ids);
	$result = $db->sql_query($sql);

	$ids = array();
	$forum_id = false;

	while ($row = $db->sql_fetchrow($result))
	{
		if ($acl_list && $row['forum_id'] && !$auth->acl_gets($acl_list, $row['forum_id']))
		{
			continue;
		}

		if ($acl_list && !$row['forum_id'] && !$auth->acl_getf_global($acl_list))
		{
			continue;
		}

		// Always add a global topic
		$ids[] = $row[$sql_id];
	}
	$db->sql_freeresult($result);

	if (!sizeof($ids))
	{
		return false;
	}

	return true;
}

/**
* Get simple post data
*/
function get_post_data($post_ids, $acl_list = false, $read_tracking = false)
{
	global $db, $auth, $config, $user;

	$rowset = array();

	if (!sizeof($post_ids))
	{
		return array();
	}

	$sql_array = array(
		'SELECT'	=> 'p.*, u.*, t.*, f.*',

		'FROM'		=> array(
			USERS_TABLE		=> 'u',
			POSTS_TABLE		=> 'p',
			TOPICS_TABLE	=> 't',
		),

		'LEFT_JOIN'	=> array(
			array(
				'FROM'	=> array(FORUMS_TABLE => 'f'),
				'ON'	=> 'f.forum_id = t.forum_id'
			)
		),

		'WHERE'		=> $db->sql_in_set('p.post_id', $post_ids) . '
			AND u.user_id = p.poster_id
			AND t.topic_id = p.topic_id',
	);

	if ($read_tracking && $config['load_db_lastread'])
	{
		$sql_array['SELECT'] .= ', tt.mark_time, ft.mark_time as forum_mark_time';

		$sql_array['LEFT_JOIN'][] = array(
			'FROM'	=> array(TOPICS_TRACK_TABLE => 'tt'),
			'ON'	=> 'tt.user_id = ' . $user->data['user_id'] . ' AND t.topic_id = tt.topic_id'
		);

		$sql_array['LEFT_JOIN'][] = array(
			'FROM'	=> array(FORUMS_TRACK_TABLE => 'ft'),
			'ON'	=> 'ft.user_id = ' . $user->data['user_id'] . ' AND t.forum_id = ft.forum_id'
		);
	}

	$sql = $db->sql_build_query('SELECT', $sql_array);
	$result = $db->sql_query($sql);
	unset($sql_array);

	while ($row = $db->sql_fetchrow($result))
	{
		if (!$row['forum_id'])
		{
			// Global Announcement?
			$row['forum_id'] = request_var('f', 0);
		}

		if ($acl_list && !$auth->acl_gets($acl_list, $row['forum_id']))
		{
			continue;
		}

		if (!$row['post_approved'] && !$auth->acl_get('m_approve', $row['forum_id']))
		{
			// Moderators without the permission to approve post should at least not see them. ;)
			continue;
		}

		$rowset[$row['post_id']] = $row;
	}
	$db->sql_freeresult($result);

	return $rowset;
}

?>
