<?php
/*
* @name quickmod.php
* @package phpBB3 cBB QuickMod
* @version $Id: quickmod.php,v1.0.3 20/07/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
* Note: This file include some functions adapted from phpBB3 original admin/mcp to work property with this project.
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class quickmod
{
	var $actions = array(
		'legend1'			=> 'OPTIONS_BASIC',
		'approve'			=> array('lang' => 'APPROVE',		'auth' => 'm_approve'),
		'delete'			=> array('lang' => 'DELETE',		'auth' => 'm_delete'),
		'move'				=> array('lang' => 'MOVE',			'auth' => 'm_move'),
		'lock'				=> array('lang' => 'LOCK',			'auth' => 'm_lock'),
		'unlock'			=> array('lang' => 'UNLOCK',		'auth' => 'm_lock'),

		'legend2'			=> 'TOPIC_TYPE',
		'make_normal'		=> array('lang' => 'MAKE_NORMAL',	'auth' => array('f_sticky', 'f_announce')),
		'make_sticky'		=> array('lang' => 'MAKE_STICKY',	'auth' => 'f_sticky'),
		'make_announce'		=> array('lang' => 'MAKE_ANNOUNCE',	'auth' => 'f_announce'),
		'make_global'		=> array('lang' => 'MAKE_GLOBAL',	'auth' => 'f_announce'),

		'legend3'			=> 'OPTIONS_ADVANCED',
		'rename'			=> array('lang' => 'RENAME',		'auth' => 'm_',	'in_list' => false,	'action' => "jQM.quickmod('rename', 'show');"),
		'chgposter'			=> array('lang' => 'CHANGE_POSTER',	'auth' => 'm_chgposter'),
		'chgicon'			=> array('lang' => 'CHANGE_ICON',	'auth' => 'f_icons'),
		'chgtime'			=> array('lang' => 'CHANGE_TIME',	'auth' => 'm_'),
	);
					
	var $forum_id		= 0;
	var $active			= false;
	var $topic_count	= 0;
	var $action			= '';

	function quickmod($forum_id = 0)
	{
		global $phpbb_root_path, $user, $template, $config, $auth;
		
		$user->add_lang('mods/quickmod/common');
		
		if(!defined('IN_INSTALL'))
		{
			$template->assign_vars(array(
				'S_QM_TYPE'				=> $config['qm_type'],
				'S_QM_VERSION'			=> $config['qm_version'],
				'T_QUICKMOD_PATH'		=> $phpbb_root_path . 'quickmod/',
			));
		}

		/* Admin initialize, no more needed */
		if($forum_id === false)
		{
			return true;
		}
		
		if(!empty($config['qm_enabled']))
		{
			$this->forum_id = request_var('f', $forum_id);
			$this->active 	= ($auth->acl_get('m_', $this->forum_id) ? true : false);
	
			if(isset($_POST['qm_ajax']))
			{
				$this->do_action();
				exit_handler();
			}
	
			$this->setup();
		}
	}

	function do_action()
	{
		global $user, $auth, $db, $template, $phpbb_root_path, $phpEx;

		if(!$this->active)
		{
			return false;
		}

		if(!function_exists('move_topics'))
		{
			include($phpbb_root_path . 'includes/functions_admin.' . $phpEx);
		}

		if(!function_exists('rename_topic'))
		{
			include($phpbb_root_path . 'quickmod/includes/functions_quickmod.' . $phpEx);
		}

		$user->add_lang('mcp');

		$this->action	= (isset($_POST['fb']) ? request_var('a', '') : request_var('quick_mod_action', ''));
		$topic_ids		= (isset($_POST['fb']) ? array(request_var('t', 0)) : request_var('topic_id_list', array(0)));

		$message = $new_data = '';
		$no_authorized_ids = array();
		
		$submit = (isset($_POST['submit_action']) ? true : false);

		if(!isset($this->actions[$this->action]) || !check_form_key('quick_mod_form'))
		{
			$this->send('FORM_INVALID', true);
		}

		if(empty($topic_ids))
		{
			$this->send('NO_TOPIC_SELECTED', true);
		}
		
		$this->topic_count = sizeof($topic_ids);

		/* If no general forum present, determine each forum individually */
		if(!$this->forum_id)
		{
			$sql = 'SELECT topic_id, forum_id
				FROM ' . TOPICS_TABLE . '
				WHERE ' . $db->sql_in_set('topic_id', $topic_ids) . '
				AND forum_id <> 0';
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				if(!$auth->acl_gets($this->actions[$this->action]['auth'], $row['forum_id']))
				{
					$no_authorized_ids[] = $row['topic_id'];
				}
			}
			$db->sql_freeresult($result);
			
			if(sizeof($no_authorized_ids))
			{
				$topic_ids = array_diff($topic_ids, $no_authorized_ids);
				
				if(empty($topic_ids))
				{
					$this->send('NO_AUTH_OPERATION', true);
				}
			}
		}
		else if(!$auth->acl_gets($this->actions[$this->action]['auth'], $this->forum_id))
		{
			$this->send('NO_AUTH_OPERATION', true);
		}

		switch($this->action)
		{
			case 'lock':
			case 'unlock':
				if($this->confirm(strtoupper($this->action)))
				{
					$message = lock_unlock($this->action, $topic_ids);
				}
			break;

			case 'make_announce':
			case 'make_sticky':
			case 'make_global':
			case 'make_normal':
				// Global topic involved?
				$global_involved = false;

				$sql = 'SELECT forum_id
					FROM ' . TOPICS_TABLE . '
					WHERE ' . $db->sql_in_set('topic_id', $topic_ids) . '
						AND forum_id = 0';
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if($row)
				{
					$global_involved = true;
				}

				if(!$submit && !$this->forum_id && $this->action != 'make_global' && $global_involved)
				{
					$template->assign_vars(array(
						'MESSAGE_TITLE'		=> $user->lang('FORUM_SELECT'),
						'S_FORUM_REQUEST'	=> true,
						'S_FORUM_SELECT'	=> make_forum_select(false, false, false, true, true, true),
					));

					$this->send('tpl');
				}
				else if($this->confirm('CHANGE', $submit))
				{
					$message = change_topic_type($this->action, $topic_ids, $this->forum_id, $global_involved);
				}
			break;

			case 'move':
				if($submit)
				{
					$message = mcp_move_topic($topic_ids, $this->forum_id);
				}
				else
				{
					$template->assign_vars(array(
						'MESSAGE_TITLE'		=> $user->lang['MOVE_TOPIC'],
						'S_FORUM_REQUEST'	=> true,
						'S_CAN_LEAVE'		=> ($this->forum_id) ? true : false,
						'S_FORUM_SELECT'	=> make_forum_select(false, $this->forum_id, false, true, true, true),
					));

					$this->send('tpl');
				}
			break;

			case 'delete':
				if($this->confirm('DELETE'))
				{
					$message = mcp_delete_topic($topic_ids);
				}
			break;

			case 'rename':
				if($this->confirm('RENAME'))
				{
					$message = rename_topic($topic_ids, $new_data);
				}
			break;

			case 'approve':
				if($submit)
				{
					$message = approve_topic($this->action, $topic_ids);
				}
				else
				{
					$template->assign_vars(array(
						'MESSAGE_TITLE'		=> $user->lang['APPROVE_TOPIC'],
						'S_APPROVE'			=> true,
					));

					$this->send('tpl');
				}
			break;
			
			case 'chgposter':
				if($submit)
				{
					$message = qm_chgposter($topic_ids);
				}
				else
				{
					$template->assign_vars(array(
						'MESSAGE_TITLE'		=> $user->lang['CHANGE_POSTER'],
						'S_CHGPOSTER'		=> true,
						'U_FIND_USERNAME'	=> append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=searchuser&amp;form=quick_mod_form&amp;field=username&amp;select_single=true'),
					));

					$this->send('tpl');
				}
			break;
			
			case 'chgicon':
				if($submit)
				{
					$message = qm_chgicon($topic_ids);
				}
				else
				{
					if(!function_exists('posting_gen_topic_icons'))
					{
						include($phpbb_root_path . 'includes/functions_posting.' . $phpEx);
					}
					
					$template->assign_vars(array(
						'MESSAGE_TITLE'		=> $user->lang['CHANGE_ICON'],
						'S_CHGICON'			=> true,
					));
					
					posting_gen_topic_icons('edit', 0);

					$this->send('tpl');
				}
			break;
			
			case 'chgtime':
				if($submit)
				{
					$message = qm_chgtime($topic_ids);
				}
				else
				{
					$month_names = array('January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');

					$s_day_options = $s_month_options = $s_year_options = '';
					
					$now = getdate();

					for ($i = 1; $i < 32; $i++)
					{
						$selected = ($i == $now['mday']) ? ' selected="selected"' : '';
						$s_day_options .= "<option value=\"$i\"$selected>$i</option>";
						
						if($i < 13)
						{
							$month_name = $user->lang['datetime'][$month_names[$i-1]];

							$selected = ($i == $now['mon']) ? ' selected="selected"' : '';
							$s_month_options .= "<option value=\"$i\"$selected>$month_name</option>";
						}
					}

					for ($i = $now['year'] - 100; $i <= $now['year']; $i++)
					{
						$selected = ($i == $now['year']) ? ' selected="selected"' : '';
						$s_year_options .= "<option value=\"$i\"$selected>$i</option>";
					}

					$template->assign_vars(array(
						'MESSAGE_TITLE'		=> $user->lang['CHANGE_TIME_TOPIC'],
						'S_CHGTIME'			=> true,
						
						'S_DAY_OPTIONS'		=> $s_day_options,
						'S_MONTH_OPTIONS'	=> $s_month_options,
						'S_YEAR_OPTIONS'	=> $s_year_options,
						'S_HOUR'			=> sprintf('%d:%d:%d', $now['hours'], $now['minutes'], $now['seconds']),
					));
					
					unset($now);

					$this->send('tpl');
				}
			break;

			default:
			break;
		}

		$this->send($message, false, $new_data);
	}

	function setup()
	{
		global $phpbb_root_path, $user, $auth, $config, $template, $phpEx;
		
		if(!$this->active)
		{
			return false;
		}

		$quick_options = array('' => 'SELECT_ACTION');

		foreach($this->actions as $action => $data)
		{
			if(strpos($action, 'legend') !== false)
			{
				$quick_options[$action] = $data;
				continue;
			}
			
			if(isset($data['in_list']) && !$data['in_list'])
			{
				continue;
			}

			if(!$this->forum_id || $auth->acl_gets($data['auth'], $this->forum_id))
			{
				$quick_options[$action] = $data['lang'] . '_TOPIC';
			}
		}
		
		$mark_options = array(
			'-1'		=> 'MARK_TOPICS',

			'legend1'	=> 'MARK_GENERAL',
				'all'		=> 'MARK_ALL',
				'none'		=> 'UNMARK_ALL',
			
			'legend2'	=> 'MARK_BY_STATUS',
				'status-' . ITEM_LOCKED		=> 'MARK_LOCKED',
				'status-' . ITEM_UNLOCKED	=> 'MARK_UNLOCKED',
				'status-' . ITEM_MOVED		=> 'MARK_MOVED',
			
			'legend3'	=> 'MARK_BY_TYPE',
				'type-' . POST_NORMAL	=> 'MARK_NORMAL',
				'type-' . POST_STICKY	=> 'MARK_STICKY',
				'type-' . POST_ANNOUNCE	=> 'MARK_ANNOUNCE',
				'type-' . POST_GLOBAL	=> 'MARK_GLOBAL',
		);

		add_form_key('quick_mod_form');

		$template->assign_vars(array(
			'CBB_QUICK_MOD'			=> $this->active,
			'S_QUICK_OPTIONS'		=> $this->_make_select($quick_options, ''),
			'S_QM_MARK_OPTIONS'		=> $this->_make_select($mark_options, '-1'),
			'U_QUICK_ACTION'		=> $this->build_current_url(),

			'S_HIDDEN_FIELDS'	=> build_hidden_fields(array(
				'f'				=> $this->forum_id
			)),
		));
	}
	
	function build_current_url($is_amp = true)
	{
		global $user;

		$obtain_params 	= array('st' => 0,	'sk' => '',	'sd' => '');
		$strip_params	= array('submit');
		$u_action 		= append_sid(generate_board_url() . '/' . $user->page['page'], false, false);

		if(($pos = strpos($u_action, '?')) !== false)
		{
			// Parse url to prevent duplicated variables
			preg_match_all('#[\?|\&](.*?)\=([^\&]+)#i', $u_action, $matches);
			foreach($matches[1] as $id => $key)
			{
				$key = urldecode($key);

				if(strpos($key, '[]') === false)
				{
					$params[$key] = $matches[2][$id];
					continue;
				}
				
				$key = str_replace('[]', '', $key);

				if(empty($params[$key]))
				{
					$params[$key] = array();
				}

				$params[$key][] = $matches[2][$id];
			}
			
			// Obtain extra POST params and add to the url
			foreach($obtain_params as $id => $value)
			{
				if(!isset($params[$id]))
				{
					$params[$id] = request_var($id, $value);
				}
			}
			
			// Delete not allowed or empty params and prepare the values
			foreach($params as $key => $value)
			{
				unset($params[$key]);
				
				if(!in_array($key, $strip_params))
				{
					if(is_array($value))
					{
						foreach($value as $id => $param)
						{
							if($param)
							{
								$subkey = $key . '[' . $id . ']';
								$params[$subkey] = $key . '%5B%5D=' . $param;
							}
						}
					}
					elseif($value)
					{
						$params[$key] = $key . '=' . $value;
					}
				}
			}
			
			$u_action = substr($u_action, 0, $pos);
			
			if(sizeof($params))
			{
				$u_action .= '?' . implode((($is_amp) ? '&amp;' : '&'), array_values($params));
			}
			
			unset($params);
		}
		
		return $u_action;
	}
	
	function get_row_fields($topic_type, $topic_status)
	{
		return build_hidden_fields(array(
			'topic_type'		=> $topic_type,
			'topic_status'		=> $topic_status,
		));
	}

	function get_row_actions($forum_id, $topic_type, $topic_status, $topic_unapproved)
	{
		global $user, $config, $auth;

		$qm_actions = '';

		if ($this->active)
		{
			$selected_actions = explode(',', $config['qm_buttons']);

			$qm_actions .= '<div class="quick_mod_buttons">';

			foreach($selected_actions as $action)
			{
				if(!isset($this->actions[$action]))
				{
					continue;
				}

				$data = $this->actions[$action];
				
				if(isset($data['in_button']) && !$data['in_button'])
				{
					continue;
				}

				if(($topic_type == POST_NORMAL && $action == 'make_normal') ||
					($topic_type == POST_STICKY && $action == 'make_sticky') ||
					($topic_type == POST_ANNOUNCE && $action == 'make_announce') ||
					($topic_type == POST_GLOBAL && $action == 'make_global') ||
					($topic_status == ITEM_LOCKED && $action == 'lock') ||
					($topic_status == ITEM_UNLOCKED && $action == 'unlock') ||
					(!$topic_unapproved && $action == 'approve') ||
					($topic_status == ITEM_MOVED && $action != 'rename' && $action != 'delete'))
				{
					continue;
				}
				
				if(!isset($data['action']))
				{
					$data['action'] = "jQM.quickmod('go', '$action')";
				}

				if($auth->acl_gets($data['auth'], $forum_id))
				{
					$qm_actions .= '<a href="#" class="qm-icon q_' . $action . '" onclick="' . $data['action'] . '; return false;" title="' . $user->lang[$data['lang'] . '_TOPIC'] . '"></a>';
				}
			}

			$qm_actions .= '</div>';
		}

		return $qm_actions;
	}
	
	function confirm($message, $submit = false)
	{
		global $user, $config;

		$is_confirmed = request_var('confirm', false);
		
		$message = 'CONFIRM_' . $message . '_TOPIC' . (($this->topic_count > 1) ? 'S' : '');
		
		if($config['qm_confirm'] && !$is_confirmed && !$submit)
		{
			$this->send(array(
				'msg'	=> (isset($user->lang[$message]) ? $user->lang[$message] : $message),
				'type'	=> 'confirm',
			));
			
			return false;
		}
		
		return true;
	}

	function send($message, $error = false, $new_data = '')
	{
		global $user, $template;

		$in_rename = false;
		
		$json = array(
			'type'		=> 'json'
		);
		
		if(is_array($message))
		{
			$json = array_merge($json, $message);
		}
		else if($message == 'tpl')
		{
			$s_hidden_fields = array();
			$s_fb = 'false';

			if(isset($_POST['fb']))
			{
				$s_fb = 'true';

				$s_hidden_fields = array(
					'fb'	=> 1,
					'a'		=> request_var('a', ''),
					't'		=> request_var('t', 0),
				);
			}

			$template->assign_vars(array(
				'DIALOG_CONTENT'		=> true,
				'S_QM_HIDDEN_FIELDS'	=> build_hidden_fields($s_hidden_fields),
				'S_FB'					=> $s_fb,
			));
			
			$json['type'] = 'html';
		}
		else
		{
			if(!$message || $message === false)
			{
				$message = 'NOTHING';
				$error = true;
			}

			if($message == 'TOPIC_RENAMED_SUCCESS')
			{
				$in_rename = true;
			}

			// Return message in JSON format
			$json += array(
				'action'	=> $this->action,
				'msg'		=> ($in_rename) ? $new_data : $user->lang($message),
				'status'	=> ($error) ? 2 : 1,
				'redirect'	=> (!$in_rename) ? $this->build_current_url(false) : '',
			);
		}
		
		$template->assign_var('JSON_RESULT', json_encode($json));

		$template->set_filenames(array('body' => 'quickmod_body.html'));

		$template->display('body');

		garbage_collection();
		exit_handler();
	}
	
	function _make_select($options, $select = false, $parse_lang = true, $empty_label = false)
	{
		global $user;

		$string = '';
		
		$in_group = false;

		foreach($options as $value => $lang)
		{
			$title		= (($parse_lang && isset($user->lang[$lang])) ? $user->lang[$lang] : $lang);

			if(strpos($value, 'legend') !== false)
			{
				$string .= (($in_group) ? '</optgroup>' : '') . '<optgroup label="' . $title . '">';
				$in_group = true;
				
				continue;
			}

			$selected	= (($value == $select) ? ' selected="selected"' : '');
			$string .= '<option value="' . $value . '"' . $selected . '>' . $title . '</option>';
		}
		
		if($in_group)
		{
			$string .= '</optgroup>';
		}
		
		if(!$string && $empty_label !== false)
		{
			$string = '<option value="0">' . (isset($user->lang[$empty_label]) ? $user->lang[$empty_label] : $empty_label) . '</option>';
		}

		return $string;
	}
}

if(!function_exists('json_encode'))
{
	function json_encode($arr)
	{
		$parts		= array();
		$is_list	= false;
		$keys		= array_keys($arr);
		$max_length	= count($arr) - 1;

		//See if the first key is 0 and last key is length - 1
		if($keys[0] == 0 && $keys[$max_length] == $max_length)
		{
			$is_list = true;

			for($i = 0; $i < count($keys); $i++)
			{
				if($i != $keys[$i])
				{
					$is_list = false; //It is an associative array.
					break;
				}
			}
		}

		foreach($arr as $key => $value)
		{
			$str = (($is_list) ? '' : "'$key':");

			//Custom handling for arrays
			if(is_array($value))
			{ 
				$parts[] = $str . json_encode($value); // RECURSION
			}
			else
			{
				//Custom handling for multiple data types
				if(is_numeric($value))
				{
					$str .= $value; //Numbers
				}
				elseif($value === true || $value === false)
				{
					$str .= ($value) ? 'true' : 'false'; //The booleans
				}
				else
				{
					$str .= '"' . addslashes($value) . '"'; //All other things
				}

				$parts[] = $str;
			}
		}

		$json = implode(',', $parts);

		//Return numerical JSON
		if($is_list)
		{
			return '[' . $json . ']';
		}

		//Return associative JSON
		return '{' . $json . '}';
	}
}

?>
